"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractGotResponse = void 0;
const ALLOWED_HEADERS = new Set(['content-type']);
function extractGotResponse(gotResponse) {
    let requestBody = gotResponse.body;
    try {
        requestBody = JSON.parse(gotResponse.body);
    }
    catch (e) {
        // pass
    }
    const filteredHeaders = Object.keys(gotResponse.headers || {}).reduce((acc, key) => {
        var _a;
        if (((_a = gotResponse.headers) === null || _a === void 0 ? void 0 : _a[key]) !== undefined && ALLOWED_HEADERS.has(key.toLowerCase())) {
            // see https://tools.ietf.org/html/rfc2616#section-4.2 we can join array to comma-separated string
            acc[key] = Array.isArray(gotResponse.headers[key])
                ? gotResponse.headers[key].join(',')
                : gotResponse.headers[key];
        }
        return acc;
    }, {});
    return {
        url: gotResponse.url,
        headers: filteredHeaders,
        status: gotResponse.statusCode,
        body: requestBody,
    };
}
exports.extractGotResponse = extractGotResponse;
