"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressTeremock = void 0;
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const ws_1 = tslib_1.__importDefault(require("ws"));
const http_1 = tslib_1.__importDefault(require("http"));
const logger_1 = tslib_1.__importDefault(require("../logger"));
const index_1 = require("./index");
const driver_1 = tslib_1.__importDefault(require("./driver"));
const __1 = require("..");
class ExpressTeremock extends __1.Teremock {
    async start(opts) {
        var _a;
        const { app, env, ...rest } = opts;
        this.driver = (_a = this.driver) !== null && _a !== void 0 ? _a : new driver_1.default({ app, env });
        return await super.start(rest);
    }
}
exports.ExpressTeremock = ExpressTeremock;
const expressTeremock = new ExpressTeremock();
const tes = {
    listen(opts = { env: {} }) {
        var _a, _b;
        const app = (_a = opts.app) !== null && _a !== void 0 ? _a : express_1.default();
        const env = opts.env;
        const wd = opts.wd;
        const port = (_b = opts.port) !== null && _b !== void 0 ? _b : index_1.DEFAULT_WS_PORT;
        const server = http_1.default.createServer(app);
        const wss = new ws_1.default.Server({ server });
        wss.on('connection', (ws) => {
            ws.on('message', async (message) => {
                const { method, data } = JSON.parse(message);
                console.log('data', data);
                console.log('message', message);
                switch (method) {
                    case 'start':
                        await expressTeremock.start({ wd, ...data, app, env, getMockId: opts.getMockId });
                        ws.send(JSON.stringify({ message: 'started' }));
                        break;
                    case 'add':
                        await expressTeremock.add(data);
                        ws.send(JSON.stringify({ message: 'added' }));
                        break;
                    case 'remove':
                        await expressTeremock.remove(data);
                        ws.send(JSON.stringify({ message: 'removed' }));
                        break;
                    case 'stop':
                        await expressTeremock.stop(data);
                        // ws.send(JSON.stringify({ message: 'stopped' }))
                        break;
                }
            });
            logger_1.default.info(`Client connected on ${port}`);
        });
        server.listen(port, () => {
            logger_1.default.info(`WebSocket start to listen on ${port}`);
        });
    },
};
exports.default = tes;
