"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("./logger");
const utils_1 = require("./utils");
const logger = logger_1.debug('teremock:response');
function createHandler(initialParams) {
    logger('creating response handler');
    return async function handleResponse({ request, response: pResponse, __meta }, extraParams = {}) {
        var _a;
        const params = { ...initialParams, ...extraParams };
        const { storage, reqSet, ci, skipResponseHeaders, skipRequestHeaders, getMockId } = params;
        utils_1.loggerTrace(`${request.url} → handling response`);
        const response = {
            ...pResponse,
            headers: utils_1.blacklist(pResponse.headers, skipResponseHeaders)
        };
        logger(`» intercepted response with method "${request.method}" and url "${request.url}"`);
        const interceptor = __meta === null || __meta === void 0 ? void 0 : __meta.interceptor;
        if (!interceptor) {
            utils_1.loggerTrace(`${request.url} → no interceptor found :(`);
            logger(`» interceptor not provided in response object, return`);
            return;
        }
        const naming = (_a = interceptor.naming) !== null && _a !== void 0 ? _a : {};
        const mockId = getMockId({ ...request, naming, name: interceptor.name, body: utils_1.getBody(request.body) });
        const mockExist = await storage.has(mockId);
        const hasResp = !!interceptor.response;
        const mog = logger_1.debug(`teremock:${mockId}`);
        /**
         * @todo update mock in non-ci mode
         */
        if (ci) {
            utils_1.loggerTrace(`${request.url} → ci is true, not storing`);
            mog(`« mock was not stored because it is CI mode run`);
        }
        else if (mockExist) {
            utils_1.loggerTrace(`${request.url} → mock already exists`);
            mog(`« mock was not stored because it exists`);
        }
        else if (hasResp) {
            utils_1.loggerTrace(`${request.url} → interceptor.response is defined`);
            mog(`« mock was not stored because matched interceptor have response property`);
        }
        else if (interceptor.pass) {
            utils_1.loggerTrace(`${request.url} → interceptor.pass is true, no store`);
            mog(`« mock was not stored because interceptor.pass is true`);
        }
        else {
            utils_1.loggerTrace(`${request.url} → storing mock ${mockId}`);
            mog(`« preparing to set a new mock "${mockId}"`);
            const { timestamp: _x, order: _z, id, ...mockRequest } = request;
            const { timestamp: _y, order: _k, ...mockResponse } = response;
            if (mockRequest.headers) {
                mockRequest.headers = utils_1.blacklist(mockRequest.headers, skipRequestHeaders);
            }
            await storage.set(mockId, { request: mockRequest, response: mockResponse });
        }
        reqSet.delete(mockId);
        mog('« reqSet after delete is', Array.from(reqSet));
        params._onReqCompleted(request, response);
        if (reqSet.size === 0) {
            mog('« invoking _onReqsCompleted');
            params._onReqsCompleted();
        }
    };
}
exports.default = createHandler;
