"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractPuppeteerRequest = void 0;
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const utils_1 = require("../utils");
const logger = debug_1.default('teremock:driver:puppeteer:request');
let requestsCounter = 0;
async function extractPuppeteerRequest(puppeteerRequest, options) {
    const request = {
        url: puppeteerRequest.url(),
        method: puppeteerRequest.method(),
        headers: puppeteerRequest.headers(),
        body: puppeteerRequest.postData(),
        resourceType: puppeteerRequest.resourceType(),
        id: requestsCounter++,
        timestamp: options.timestamp,
        order: options.order,
    };
    // @ts-expect-error
    puppeteerRequest.timestamp = options.timestamp;
    // @ts-expect-error
    puppeteerRequest.teremockRequest = request;
    // @ts-expect-error
    puppeteerRequest.__meta = { request };
    logger(`got the request ${request.url}, sending it to teremock core`);
    return {
        request,
        abort: async (errorCode) => await puppeteerRequest.abort(errorCode),
        next: async (interceptor) => {
            logger(`continue() call`);
            // @ts-expect-error
            puppeteerRequest.__meta.interceptor = interceptor;
            utils_1.loggerTrace(`${request.url} ← request.continue()`);
            await puppeteerRequest.continue();
        },
        respond: async (response, interceptor) => {
            logger(`respond() call`);
            // @ts-expect-error
            puppeteerRequest.__meta.interceptor = interceptor;
            utils_1.loggerTrace(`${request.url} ← request.respond(...)`);
            await puppeteerRequest.respond(response);
        },
    };
}
exports.extractPuppeteerRequest = extractPuppeteerRequest;
