"use strict";
/*
 * It is a singleton, the only instance is located in `exports`
 * @todo instantiate for each test suite explicitly
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const storage_1 = tslib_1.__importDefault(require("./storage"));
const logger_1 = tslib_1.__importDefault(require("./logger"));
const handleRequest_1 = tslib_1.__importDefault(require("./handleRequest"));
const handleResponse_1 = tslib_1.__importDefault(require("./handleResponse"));
const utils_1 = require("./utils");
const p_driver_1 = tslib_1.__importDefault(require("./p-driver"));
const consts_1 = require("./consts");
const logger = debug_1.default('teremock');
function noop() { }
class Teremock {
    constructor(opts) {
        var _a;
        logger_1.default.debug(`new Mocker`);
        this.defaultOptions = Object.assign({}, consts_1.DEFAULT_OPTIONS);
        this.options = this.defaultOptions;
        this.alive = false;
        this._spies = [];
        this._interceptors = {};
        this._resolveReqs = noop;
        this._matched = new Map();
        this.storage = (_a = opts === null || opts === void 0 ? void 0 : opts.storage) !== null && _a !== void 0 ? _a : new storage_1.default();
        this.driver = opts === null || opts === void 0 ? void 0 : opts.driver;
        this.reqSet = new Set();
        this.reqSetRet = new Set();
    }
    _onAnyReqStart(req) {
        if (this.reqSet.size === 0) {
            this.reqsPromise = new Promise((resolve) => {
                this._resolveReqs = () => {
                    resolve(Array.from(this.reqSetRet));
                    this.reqSetRet.clear();
                };
            });
        }
        if (this._spies.length > 0) {
            this._spies.forEach(([interceptor, spy]) => {
                if (utils_1.isInterceptorMatched(interceptor, req)) {
                    spy.called = true;
                    spy.callCount++;
                    spy.calledOnce = spy.callCount === 1;
                    if (req.id !== -1 && req.id && req.order && req.timestamp) {
                        spy.events.push({
                            requestTimestamp: req.timestamp,
                            requestOrder: req.order,
                            requestId: req.id
                        });
                    }
                }
            });
        }
    }
    _onAnyReqEnd(req, resp) {
        if (this._spies.length > 0) {
            this._spies.forEach(([interceptor, spy]) => {
                if (utils_1.isInterceptorMatched(interceptor, req)) {
                    if (req.id !== -1 && req.id) {
                        const eventInfo = spy.events.find(obj => obj.requestId === req.id);
                        if (eventInfo) {
                            eventInfo.responseTimestamp = resp.timestamp;
                            eventInfo.responseOrder = resp.order;
                        }
                    }
                }
            });
        }
    }
    /*
     * Starts to intercept requests
     */
    async start(userOptions) {
        var _a, _b, _c, _d, _e;
        logger_1.default.debug(`mocker.start()`);
        if (this.alive) {
            logger_1.default.warn(`mocker was not stopped before new start call, stopping it first`);
            await this.stop();
        }
        let resolveStartPromise;
        this._startPromise = new Promise((resolve) => {
            resolveStartPromise = resolve;
        });
        this.alive = true;
        this.options = utils_1.userOptionsToOptions(this.defaultOptions, userOptions);
        this.options.wd && ((_b = (_a = this.storage).setWd) === null || _b === void 0 ? void 0 : _b.call(_a, this.options.wd));
        this.driver = (_c = this.driver) !== null && _c !== void 0 ? _c : new p_driver_1.default({ page: userOptions.page });
        await this.driver.setRequestInterception(true);
        this._spies = [];
        this._interceptors = this.options.interceptors;
        this._matched = new Map();
        const logParams = Object.assign({}, this.options);
        logger('Mocker starts with resulting params:', logParams);
        // Clear on any page close, or sometimes you loose some responses on unload, and `connections` will never resolves
        this.removeCloseHandler = (_e = (_d = this.driver).onClose) === null || _e === void 0 ? void 0 : _e.call(_d, () => {
            logger('removeCloseHandler', this.reqSet.size);
            if (this.reqSet.size !== 0) {
                if (!this.options.ci) {
                    logger_1.default.error(`Some connections was not completed, but navigation happened.`);
                    logger_1.default.error(`That is bad, mkay? Because you have a race: server response and navigation`);
                    logger_1.default.error(`That will lead to heisenberg MONOFO errors in case when response will win the race`);
                    logger_1.default.error(`Alive connections:\n${[...this.reqSet]}`);
                }
                this.reqSet.clear();
                this._resolveReqs();
            }
            this.stop({ safe: true });
        });
        const pureRequestHandler = handleRequest_1.default({
            ...this.options,
            interceptors: this._interceptors,
            storage: this.storage,
            reqSet: {
                add: (en) => {
                    this.reqSet.add(en);
                    this.reqSetRet.add(en);
                },
                get: () => this.reqSet,
            },
            _onReqStarted: (req) => this._onAnyReqStart(req),
            _onMatch: (interceptor, req) => {
                if (!this._matched.has(interceptor.name)) {
                    this._matched.set(interceptor.name, []);
                }
                this._matched.get(interceptor.name).push(req.url);
            }
        });
        this._requestPendingQueue = new utils_1.AsyncPendingQueue();
        const pureResponseHandler = handleResponse_1.default({
            ...this.options,
            interceptors: this._interceptors,
            storage: this.storage,
            reqSet: this.reqSet,
            _onReqCompleted: (req, resp) => this._onAnyReqEnd(req, resp),
            _onReqsCompleted: () => this._resolveReqs(),
        });
        this._responsePendingQueue = new utils_1.AsyncPendingQueue();
        logger('Handlers created, params validated');
        // Intercepting all requests and respinding with mocks
        this.removeRequestHandler = await this.driver.onRequest(async (ir) => {
            // driver may calls onRequest callback synchronously,
            // we using here AsyncPendingQueue who can track pending promises
            try {
                await this._requestPendingQueue.add(pureRequestHandler(ir, {
                    ...this.extraParams,
                    interceptors: this._interceptors,
                }));
            }
            catch (e) {
                logger_1.default.error(`There is error in request handler:`);
                logger_1.default.error(e);
            }
        });
        // Writing mocks on real responses to filesystem
        this.removeResponseHandler = await this.driver.onResponse(async (ir) => {
            // driver may calls onResponse callback synchronously,
            // we using here AsyncPendingQueue who can track pending promises
            try {
                await this._responsePendingQueue.add(pureResponseHandler(ir, {
                    ...this.extraParams,
                    interceptors: this._interceptors,
                }));
            }
            catch (e) {
                logger_1.default.error(`There is error in response handler:`);
                logger_1.default.error(e);
            }
        });
        logger('_startPromise about to resolve (Request interception enabled, listeners added)');
        resolveStartPromise();
        return this._startPromise;
    }
    add(userInterceptor, overwrite = false) {
        const interceptor = utils_1.userInterceptorToInterceptor(userInterceptor);
        const { name } = interceptor;
        if (name in this._interceptors && !overwrite) {
            logger_1.default.error(`interceptor with name "${name}" already exists, pass second arg true if you want to overwrite it`);
            return () => { };
        }
        this._interceptors = {
            [name]: interceptor,
            ...this._interceptors,
        };
        // @todo support restoring overwritten names
        return () => {
            const { [name]: omit, ...rest } = this._interceptors;
            this._interceptors = rest;
        };
    }
    remove(userInterceptor) {
        const interceptor = utils_1.userInterceptorToInterceptor(userInterceptor);
        const { name } = interceptor;
        if (!this._interceptors[name]) {
            logger_1.default.error(`there is no interceptor with name "${name}"`);
        }
        const { [name]: _x, ...rest } = this._interceptors;
        this._interceptors = rest;
    }
    /*
     * Resolves when all mocked connections are completed
     * @deprecated
     */
    connections(opts = { timeout: 1000 }) {
        logger_1.default.warn(`teremock.connections() is deprecated`);
        logger_1.default.warn(`it is recommended to await explicit effects as a result of requests`);
        const { timeout } = opts;
        if (typeof this._startPromise === 'undefined') {
            throw new Error('Cant await connections. Probably you didnt start the mocker?');
        }
        const timeoutPromise = new Promise((_resolve, reject) => {
            setTimeout(() => reject(new Error(`failed to await connectins in ${timeout} ms`)), timeout);
        });
        if (typeof this.reqsPromise === 'undefined') {
            throw new Error(`no connections, nothing to await`);
        }
        return Promise.race([timeoutPromise, this.reqsPromise]);
    }
    spy(spyInterceptor) {
        const interceptor = utils_1.userInterceptorToInterceptor(spyInterceptor, 'spy');
        const spy = {
            callCount: 0,
            calledOnce: false,
            called: false,
            dismiss: () => {
                this._spies = this._spies.filter(([_i, s]) => s !== spy);
            },
            events: [],
        };
        this._spies.push([interceptor, spy]);
        return spy;
    }
    /*
     * Waits for all connections to be completed and removes all handlers from the page
     */
    async stop(opts = {}) {
        var _a, _b, _c, _d, _e, _f, _g;
        const { safe } = opts;
        if (!this.alive) {
            if (safe) {
                return;
            }
            // Async, because jest suppress Errors in `after` callbacks
            setTimeout(() => {
                logger_1.default.warn(`nothing to stop. Did you call 'mocker.stop()' twice?`);
            });
            return;
        }
        const t1 = setTimeout(() => {
            if (this.reqSet.size === 0) {
                logger(`Mocker failed to stop. reqSet.size === 0`);
            }
            else {
                logger(`Mocker failed to stop. Alive connections:\n${[...this.reqSet]}`);
            }
            logger('About to throw an Error');
            logger_1.default.error(`Failed to stop mocker!`);
            logger_1.default.error(`Possible reasons:`);
            logger_1.default.error(`1) navigation happened before all responses completed.`);
            logger_1.default.error(`2) some connections are not finished in 20 seconds.`);
            logger_1.default.error(`The number of opened connections is ${this.reqSet.size}.\nReqSet:${[...this.reqSet]}`);
            throw new Error(`Failed to stop mocker!`);
        }, 20 * 1000);
        logger('Begining of mocker.stop procedure');
        if (typeof this._startPromise === 'undefined') {
            throw new Error('Cant stop mocker. Probably you didnt start it?');
        }
        let failed = false;
        await this._startPromise;
        if (this.options.awaitConnectionsOnStop) {
            try {
                await this.connections();
            }
            catch (err) {
                logger_1.default.error(`this.connections was rejected with error. Continue...`, err);
                failed = err;
            }
        }
        logger('`Start` promise was resolved, `connections` promise was resolved or rejected');
        clearTimeout(t1);
        const requestHandlerErrorsCount = await ((_a = this._requestPendingQueue) === null || _a === void 0 ? void 0 : _a.awaitPending());
        if (requestHandlerErrorsCount) {
            throw new Error(`There were ${requestHandlerErrorsCount} errors in the request handler. See error output above.`);
        }
        const responseHandlerErrorsCount = await ((_b = this._responsePendingQueue) === null || _b === void 0 ? void 0 : _b.awaitPending());
        if (responseHandlerErrorsCount) {
            throw new Error(`There were ${responseHandlerErrorsCount} errors in the response handler. See error output above.`);
        }
        await ((_c = this.removeCloseHandler) === null || _c === void 0 ? void 0 : _c.call(this));
        await ((_d = this.removeRequestHandler) === null || _d === void 0 ? void 0 : _d.call(this));
        await ((_e = this.removeResponseHandler) === null || _e === void 0 ? void 0 : _e.call(this));
        this.alive = false;
        // @todo how to solve that without closure?
        // Must reject on `connections` reject, but must finish all tasks also
        if (failed) {
            logger_1.default.error(`rejecting mocker.stop() promise, because connections promise was rejected`);
            logger(`About to exit from mocker.stop with reject`);
            throw failed;
        }
        logger('Interceptor names and matched urls', this._matched);
        (_g = (_f = this.options).onStop) === null || _g === void 0 ? void 0 : _g.call(_f, { matched: this._matched });
        logger(`about to exit from mocker.stop with resolve`);
    }
}
exports.default = Teremock;
